import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.io.*;


class Polyvector {
    int n;
    Vector[] v=new Vector[50];

    Polyvector(int S) {
	this.n=S;
	for(int i=1;i<=S;++i) v[i]=new Vector();
    }


    Polyvector entail(Vector v,int n,double w,double h) {
	Polyvector P=new Polyvector(n);
	P.v[1]=v;
	for(int i=1;i<n;++i) {
	    P.v[i+1]=P.v[i].next(w,h);
	}
	P.v[n+1]=P.v[1];
	P.v[n+2]=P.v[2];
	P.n=n;
	return(P);
    }

    public Path2D.Double toPath() {
	Path2D.Double gp=new Path2D.Double(Path2D.WIND_EVEN_ODD);
	gp.moveTo((float)(v[1].x),(float)(v[1].y));
	for(int i=2;i<=n;++i) {
	   gp.lineTo((float)(v[i].x),(float)(v[i].y));
	}
	gp.closePath();
	return(gp);
    }

    int crossTest() {
	Vector v1=v[1];
	Vector v2=v[2];
	Vector v3=v[n-1];
	Vector v4=v[n];
	Vector w=v1.cross(v1.cross(v1,v2),v1.cross(v3,v4));
	w=w.normalize();
	int test=0;
	if((w.x>v1.x)&&(w.x<v2.x)) test=1;
        if((w.x<v1.x)&&(w.x>v2.x)) test=1;
	return(test);
    }

    double closingDistance() {
	Vector v1=v[1];
	Vector v2=v[n];
	double d=(v1.x-v2.x)*(v1.x-v2.x);
	d=d+(v1.y-v2.y)*(v1.y-v2.y);
	d=Math.sqrt(d);
	return(d);
    }

    /**center of mass routines*/

    public Vector CM(int choice) {
	if(choice==0) return(vtxCM());
	if(choice==1) return(edgCM());
	if(choice==2) return(facCM());
	if(choice==3) return(circumcenter());
	return null;
    }

    public Vector vtxCM() {

	Vector V=new Vector(0,0,0);
	for(int i=1;i<n;++i) {
	    Vector W=this.v[i].normalize();
	    V=V.plus(W);
	}
	V.x=V.x/(n-1);
	V.y=V.y/(n-1);
	V.z=1;
	return(V);
    }

    public Vector edgCM() {
	Vector V=new Vector(0,0,0);
	double D=0;
	for(int i=1;i<n;++i) {
	    int j=i+1;
	    if(j==n) j=1;
	    Vector V1=this.v[i].normalize();
	    Vector V2=this.v[j].normalize();
	    Vector V3=V2.plus(V1);  
            double d=V1.dist(V2);
	    D=D+d;
	    V3=new Vector(d*V3.x/2,d*V3.y/2,1);
	    V=V.plus(V3);
	}
	V.x=V.x/D;
	V.y=V.y/D;
	V.z=1;
	return(V);
    }



    public Vector sym2CM() {
	Complex z=new Complex(0,0);

	for(int i=1;i<n;++i) {
	    for(int j=i+1;j<n;++j){
	    Vector V1=this.v[i].normalize();
	    Vector V2=this.v[j].normalize();
	    Complex z1=new Complex(V1.x,V1.y);
	    Complex z2=new Complex(V2.x,V2.y);
	    z=Complex.plus(z,Complex.times(z1,z2));
	    }
	}   
        double D=n*(n+1)/2;
	Vector V=new Vector(z.x/D,z.y/D,1);
	return(V);
    }

    public Vector facCM() {
	Vector V=new Vector(0,0,0);
	double D=0;
	for(int i=1;i<n-2;++i) {
	    int j=i+1;
	    int k=i+2;
            Vector V1=this.v[1].normalize();
	    Vector V2=this.v[j].normalize();
	    Vector V3=this.v[k].normalize();
	    Vector V4=V3.plus(V2.plus(V1));
	    double d=V4.tripleProduct(V1,V2,V3);
	    D=D+d;
            V4=new Vector(d*V4.x/6,d*V4.y/6,1);
	    V=V.plus(V4);
	}
	V.x=2*V.x/D;
	V.y=2*V.y/D;
	V.z=1;
	return(V);
    }


    public Vector circumcenter() {
	Vector v0=this.v[1].normalize();
	Vector v1=this.v[2].normalize();
	Vector v2=this.v[3].normalize();
	Vector v01=v0.plus(v1);
	v01=v01.scale(2);
	Vector v12=v1.plus(v2);
	v12=v12.scale(2);

	v01.print();
	v12.print();

       
	Vector w01=v0.minus(v1);
	w01=new Vector(w01.y,-w01.x,w01.z);
	Vector w12=v1.minus(v2);
	w12=new Vector(w12.y,-w12.x,w12.z);
	Vector x01=v01.plus(w01);
	Vector x12=v12.plus(w12);
	x01.z=1;
	x12.z=1;
    }



    

    /**sums of cosines of angles*/

    public double cosAngleAt(int i1) {
	int K=n-1;
	int i0=(i1+K-1)%K;
	int i2=(i1+K+1)%K;
	Vector A=v[i0+1];
	Vector B=v[i1+1];
	Vector C=v[i2+1];
	B=new Vector(0,0,1);
	return Vector.cosAngle(A,B,C);
    }

    public double cosAngle2At(int i1) {
	int K=n-1;
	int i0=(i1+K-1)%K;
	int i2=(i1+K+1)%K;
	int i3=(i1+K+2)%K;
	Vector A0=v[i0+1];
	Vector A1=v[i1+1];
	Vector A2=v[i2+1];
	Vector A3=v[i3+1];
	Vector B=new Vector(0,0,1);
	double x1=Vector.angle(A0,B,A2);
	double x2=Vector.angle(A1,B,A3);
	return Math.cos(x1+x2);

    }

    public double cosAngle3At(int i1) {
	int K=n-1;
	int i0=(i1+K-1)%K;
	int i2=(i1+K+1)%K;
	int i3=(i1+K+2)%K;
	int i4=(i1+K+3)%K;
	Vector A0=v[i0+1];
	Vector A1=v[i1+1];
	Vector A2=v[i2+1];
	Vector A3=v[i3+1];
	Vector A4=v[i4+1];
	Vector B=new Vector(0,0,1);
	double x1=Vector.angle(A0,B,A2);
	double x2=Vector.angle(A1,B,A3);
	double x3=Vector.angle(A2,B,A4);
	return Math.cos(x1+x2+x3);

    }




    public double cosAngleSum() {
	double tot=0;
	for(int i=0;i<n-1;++i) {     //n-1 is the length
          double t=cosAngle2At(i);
	  tot=tot+t;
	}
	return tot;
    }

    /**print*/
    public void print() {
	System.out.println("polygon");
	for(int i=1;i<=n;++i) {
	    this.v[i].print();
	}
    }

}



